@@ -9,7 +9,8 @@ use warnings;
 sub D_PATTERN () { 0 }
 sub D_RULES () { 0 }
 sub D_EQUIV () { 0 }
-sub D_TRACE () { 1 }
+sub D_OPERATOR () { 0 }
+sub D_TRACE () { 0 }
 
 # Preamble
 
@@ -23,17 +24,19 @@ our @EXPORT = qw( declare rule unknown stringify
 		);
 for (0..9) { push @EXPORT, "_$_" } # all the shortened capture sequences
 
-our $VERSION = '0.00104'; # increment _after_ each distribution
+our $VERSION = '0.00105'; # increment _after_ each distribution
 
 use Carp;
 use YAML;
 use Scalar::Util qw( blessed );
 
 # Submodules
+use Grapefruit::Atom;
 use Grapefruit::Compound;
 use Grapefruit::Rule;
 use Grapefruit::Unknown;
 use Grapefruit::Pattern;
+use Grapefruit::Capture;
 
 # Code below
 
@@ -41,27 +44,9 @@ our @captures;
 
 sub capture ($;$) {
   my ($num, $matcher) = @_;
-  my $closure;
-  if ($matcher) {
-    $closure = sub {
-      my ($parsetree) = @_; # take the parsetree
-      if ($matcher->($parsetree)) { # if it matches the pattern
-	$captures[$num] = $parsetree; # save it
-	return 1;
-      } else {
-	return 0;
-      }
-    };
-  } else {
-    $closure = sub {
-      my ($parsetree) = @_; # take the parsetree
-      $captures[$num] = $parsetree; # save it
-      return 1; # we matched
-    };
-  }
-  return bless $closure, 'pattern::capture';
+  return Grapefruit::Capture->new($num, $matcher);
 }
-sub _0 (;$) { &capture(0, @_) }
+sub _0 (;$) { &capture(0, @_) } # the & does magic, do not remove
 sub _1 (;$) { &capture(1, @_) }
 sub _2 (;$) { &capture(2, @_) }
 sub _3 (;$) { &capture(3, @_) }
@@ -96,7 +81,7 @@ sub _do_match {
   } elsif (ref $patterntree eq "ARRAY" and
       ref $parsetree eq "ARRAY") {
     return _do_match_apply($patterntree, $parsetree);
-  } elsif (ref $patterntree eq "pattern::capture") {
+  } elsif (ref $patterntree eq "Grapefruit::Capture") {
     # probably just handle all code refs like this
     D_PATTERN and print "*** calling the pattern::capture closure with $parsetree\n";
     return $patterntree->($parsetree);
@@ -126,6 +111,9 @@ sub rule ($$$) {
   my ($prec, $match, $func) = @_;
   croak "precedence must be numeric" unless $prec =~ /^\d+$/;
 
+  D_RULES and print "Creating rule at precedence $prec:\n";
+  D_RULES and print stringify($match), " <= ", stringify($func), "\n";
+
   push @{$rules{$prec}}, Grapefruit::Rule->new($match, $func);
 }
 
@@ -180,7 +168,7 @@ sub declare ($) {
   my $code = <<"END";
     package $pkg;
     sub $name {
-      Grapefruit::run_rules(Grapefruit::Compound->new(q{$name}, \@_));
+      Grapefruit::run_rules(Grapefruit::Compound->new(q{${pkg}::$name}, \@_));
     }
 END
   eval $code;
@@ -194,6 +182,8 @@ sub stringify {
       $_->stringify;
     } elsif (ref $_ eq "ARRAY") { # this is a function application atm
       "($$_[0] ".join(" ", map stringify($_), @$_[1..$#$_]).")";
+    } elsif (not defined $_) {
+      "undef"
     } else {
       "'$_'"
     }
@@ -214,16 +204,22 @@ sub is_atom ($) {
 # bah! It'll work!
 sub equiv {
   my ($left, $right) = @_;
+  my $rv;
   D_EQUIV and printf "equiv: [%s] [%s]\n", stringify($left), stringify($right);
   if (blessed $left and $left->can('equiv')) {
-    return $left->equiv($right);
+    $rv = $left->equiv($right);
   } elsif (blessed $right and $right->can('equiv')) {
-    return $right->equiv($left);
+    $rv = $right->equiv($left);
+  } elsif (blessed $left or blessed $right) {
+    die "left or right is blessed but doesn't handle equiv ($left,$right)";
+  # randomly works, this does!
   } elsif (is_num $left and is_num $right) {
-    return $left == $right;
+    $rv = $left == $right;
   } else {
-    return $left eq $right;
+    $rv = $left eq $right;
   }
+  D_EQUIV and print "    => rv = $rv\n";
+  return $rv;
 }
 
 1;
@@ -236,13 +232,71 @@ Grapefruit - Pattern matching and reduction engine for a perl CAS
 =head1 SYNOPSIS
 
   use Grapefruit;
-  print Solve( Equals( 5, Add( 3, x ) ) );
+  print Solve( Equals( 5, Add( 3, x ) ), x );
 
 =head1 DESCRIPTION
 
-Just read the source, the comments try to be amusing.
+Just read the source, the comments try to be amusing. The only existing example
+code is in grapefruit.pl.
 
-Blah blah blah.
+Grapefruit provides a pattern matching engine which will match against a
+parsetree of an expression. The parsetree (and pattern) is generated by small
+stub functions which create a Grapefruit::Compound object using their name and
+their arguments. They then pass this into the rule based reduction engine. A
+reduction engine, in contrast to a production engine where a set of rules are
+given for producing all the valid theorems of a system (to mix terminologies
+from parsing and Post's symbolic manipulation systems) , such as a Yacc or
+Parse::RecDescent grammar, has a set of rules (sorted by precendence in this
+implementation) each of which has a predicate, and a replacement parsetree.
+When the predicate (being passed a parsetree as its argument) is true, the
+parsetree is replaced by the one specified in the replacement. Portions of the
+parsetree being matched against can be 'captured' as in normal regular
+expressions, and can be substituted into the replacement tree. In Grapefruit,
+since parsetrees are generated as the return values of functions, the
+replacements are implemented as subroutine references which are called with the
+captures are arguments.
+
+The idea is that the core of Grapefruit, the reduction engine, will stay very
+general and applicable to any application and that the ability to have
+different sets of rules will be aided by having a Grapefruit::Ruleset object.
+This means that since it's dependent on multimethods, namespace hell will
+avoided by being able to say $ruleset->import, to import all the symbols from
+your chosen ruleset or package (%pkg::RULES).
+
+Another important part is the original aim of developing a computer algebra
+system in perl (and using perl, note the difference). There's two goals here,
+one is to create a cmdline application like yacas or maxima (and optionally a
+gui via TeXmacs or similar), and the other is to enable algebra in a normal
+perl program.
+
+Here's some dream code from the distant future:
+
+  use Grapefruit::Algebra; # basic algebra rules and operators
+  use Grapefruit::Calculus::Differential; # the differential calculus, this is mainly here for fun
+  use Grapefruit::Algorithm::NewtonRaphson; # this includes the above
+  # they all export their rulesets by default; ie they have @ISA=qw(Grapefruit::Ruleset);
+
+  sub f {
+    my $x = shift;
+    return 5*$x**5+4*$x**4+5; # any takers for solving this by hand?
+  }
+
+  print Newton(\&f, 4); # this differentiates it for us!
+
+Ok that wasn't very enlightening code, and all those imports weren't needed but
+it should serve as a goal to achieve.
+
+=head1 BUGS
+
+Now I'm using objects with overloaded operators I've got a nice problem with
+complex numbers.  Because evaluation is no longer delayed we tend to end up
+with 'NaN'.  I can't simply use Math::Complex because it doesn't play well with
+Math::BigFloat which is a potential problem for this module.  I anticipate that
+I'll solve this in one of three ways: write my own complex implementation
+(which plays nicely with the whole system; turn all numbers into
+Grapefruit::Atoms possibly using constant overloading; write some code which
+detects which modules the user is using and adapt.  None of this seems much
+fun.  The second idea currently sounds the most feasible.
 
 =head1 AUTHOR
 
