@@ -10,10 +10,12 @@ use Grapefruit;
 # Preamble
 
 # we might need a version here
-our $VERSION = '0.00104';
+our $VERSION = '0.00105';
 
 use Carp;
 
+use base qw(Grapefruit::Atom);
+
 # Code below
 
 my $unique;
@@ -30,4 +32,13 @@ sub stringify {
   "{$$self}";
 }
 
+sub equiv {
+  my ($self, $right) = @_;
+  if (UNIVERSAL::isa($right, 'Grapefruit::Unknown')) {
+    return $$self == $$right; # do the ids match?
+  } else {
+    return 0;
+  }
+}
+
 1;
